﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using PcmCore.Donnees;
using PcmCore.Event;
using PcmCore.Metier;

namespace Pcm.Gui
{
    public partial class MainWindow : Form
    {
        private bool isConnected;
        private List<Contact> allContacts;
        private Contact contactConsulte;
        private ContactsManager contactSession;
        private User utilisateurSession;

        public MainWindow ()
        {
            isConnected = false;
            allContacts = new List<Contact> ();
            InitializeComponent ();
            this.Text = PcmCore.CoreInformations.CoreVersion;
            updateInterfaceState();
        }

        private void MainWindow_Load (object sender, EventArgs e)
        {
            
        }

        /// <summary>
        /// Lancement de la fenêtre de connexion des utilsateurs
        /// </summary>
        private void lancerConnexionUtilisateur ()
        {
            UserConnectionWindow nouvelleConnexion = new UserConnectionWindow ();
            nouvelleConnexion.utilisateurConnecte += this.event_UtilisateurConnecte;
            nouvelleConnexion.Show ();
        }

        /// <summary>
        /// Cette méthode est chargée de mettre à jour l'état de l'interface en utilisant le membre privé
        /// isConnected
        /// </summary>
        private void updateInterfaceState()
        {
            if (isConnected)
            {
                modifierLutilisateurToolStripMenuItem.Enabled = true;
                menuContacts.Enabled = true;
                splitContainerMW.Show ();
                listContacts.Items.Clear ();
                allContacts = contactSession.AllContacts ();

                foreach (Contact c in allContacts)
                    listContacts.Items.Add (String.Format ("{0} {1}", c.Nom, c.Prenom));

                etatConnexion.Enabled = true;
                barreStatutLabel.Text = String.Format ("{0} est connecté(e)", utilisateurSession.NomUtilisateur);
            }
            else
            {
                utilisateurSession = null;
                contactSession = null;
                modifierLutilisateurToolStripMenuItem.Enabled = false;
                menuContacts.Enabled = false;
                splitContainerMW.Hide ();
                etatConnexion.Enabled = false;
                barreStatutLabel.Text = "Aucun utilisateur n'est connecté";
                allContacts.Clear ();
            }
        }

        /// <summary>
        /// Affiche un contact dans la zône d'affichage
        /// </summary>
        /// <param name="c">Un objet contact</param>
        private void afficherContact(Contact c)
        {
            int idC = c.Id;
            contactConsulte = c;
            txtNom.Text = c.Nom;
            txtPrenom.Text = c.Prenom;
            String[] s = c.DateNaissance.ToString().Split('/');
            if (s.Length == 3)   
                dpDateNaissance.Value = new DateTime (int.Parse (s[2]), int.Parse (s[1]), int.Parse (s[0]));
            txtAdresse.Text = c.Adresse;
            txtCodePostal.Text = c.CodePostal;
            txtVille.Text = c.Ville;
            txtTfixe.Text = c.NumTelFix;
            txtTPort.Text = c.NumTelPort;
            txtMail.Text = c.AdrMail;
        }
       
        #region Evénements Menu Fichier

        // --- Menu Fichier ---

        /// <summary>
        /// Connexion d'un utilisateur
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void itemConnexion_Click (object sender, EventArgs e)
        {
            if (isConnected)    // Si déjà connecté on propose une déconnexion
            {
                DialogResult reponse = MessageBox.Show ("Vous êtes déjà connecté, voulez vous vous déconnecter ?", "Connexion", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                if (reponse == DialogResult.Yes)
                {
                    isConnected = false;
                    btEffacerContact_Click (this, new EventArgs ());        // Vide la zone de contact
                    updateInterfaceState();
                    lancerConnexionUtilisateur ();
                }
            }
            else
                lancerConnexionUtilisateur ();
            
        }

        /// <summary>
        /// Déconnexion d'un utilisateur
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void itemDeconnexion_Click (object sender, EventArgs e)
        {
            if (isConnected)
            {
                DialogResult reponse = MessageBox.Show ("Etes vous sur de vous vous déconnecter ?", "Déconnexion", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (reponse == DialogResult.Yes)
                {
                    isConnected = false;
                    btEffacerContact_Click (this, new EventArgs ());        // Vide la zone de contact
                    updateInterfaceState ();
                }
            }
            else
                MessageBox.Show ("Aucun utilisateur n'est connecté", "Déconnexion", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        /// <summary>
        /// Quitter l'application
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void itemQuitter_Click (object sender, EventArgs e)
        {
            if (isConnected)
            {
                isConnected = false;
                updateInterfaceState ();
            }
            Application.Exit ();
        }
        #endregion

        #region Evénéments Menu Utilisateur
        /// <summary>
        /// Création d'un utilisateur
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void itemCreerUtilisateur_Click (object sender, EventArgs e)
        {
            AddUserWindow wModUtil = new AddUserWindow ();
            wModUtil.utilisateurAjoute += this.event_UtilisateurAjoute;
            wModUtil.Show ();
        }

        private void modifierLutilisateurToolStripMenuItem_Click (object sender, EventArgs e)
        {
            UserEditorWindow editUtilisateur = new UserEditorWindow (utilisateurSession);
            editUtilisateur.Show ();
        }
        #endregion

        #region Evénements Menu contacts
        private void ajouterUnContactToolStripMenuItem_Click (object sender, EventArgs e)
        {
            AddContactWindow wAddContact = new AddContactWindow (utilisateurSession);
            wAddContact.contactAjoute += this.event_ContactAjoute;
            wAddContact.Show ();
        }

        private void versUnFichierXMLToolStripMenuItem_Click (object sender, EventArgs e)
        {
            SaveFileDialog dialogSave = new SaveFileDialog ();
            dialogSave.DefaultExt = "xml";
            dialogSave.Filter = "Fichier XML (*.xml)|*.xml|Tout types (*.*)|*.*";
            dialogSave.AddExtension = true; // ajoute une extension automatiquement
            dialogSave.RestoreDirectory = true; // Restaure le dernier dossier choisi
            dialogSave.Title = "Où voulez vous sauvegarder vos contacts ?";
            dialogSave.InitialDirectory = @"C:\";

            if (dialogSave.ShowDialog () == DialogResult.OK)
            {
                ContactsExporter exporter = new ContactsExporter (contactSession, dialogSave.FileName);
                exporter.ExportToXML ();
                if (System.IO.File.Exists (dialogSave.FileName))
                    MessageBox.Show ("Vos contacts ont été sauvegardés", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                else
                    MessageBox.Show ("Erreur lors de la sauvegarde de vos contacts", "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            dialogSave.Dispose ();
            dialogSave = null;
        }

        private void versUnFichierTexteToolStripMenuItem_Click (object sender, EventArgs e)
        {
            SaveFileDialog dialogSave = new SaveFileDialog ();
            dialogSave.DefaultExt = "txt";
            dialogSave.Filter = "Fichier Texte (*.txt)|*.txt|Tout types (*.*)|*.*";
            dialogSave.AddExtension = true; // ajoute une extension automatiquement
            dialogSave.RestoreDirectory = true; // Restaure le dernier dossier choisi
            dialogSave.Title = "Où voulez vous sauvegarder vos contacts ?";
            dialogSave.InitialDirectory = @"C:\";

            if (dialogSave.ShowDialog () == DialogResult.OK)
            {
                ContactsExporter exporter = new ContactsExporter (contactSession, dialogSave.FileName);
                exporter.ExportToTextFormat ();
                if (System.IO.File.Exists (dialogSave.FileName))
                    MessageBox.Show ("Vos contacts ont été sauvegardés", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                else
                    MessageBox.Show ("Erreur lors de la sauvegarde de vos contacts", "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            dialogSave.Dispose ();
            dialogSave = null;
        }
        #endregion

        #region Evénénements Menu Aide
        private void aProposDePCMToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string messageAbout = "Personal Contacts Manager est un gestionnaire de contacts gratuit";
            messageAbout += "\nCe programme vous permet de mémoriser vos contacts rapidement et facilement";
            messageAbout += "\nYannick Comte - 2010";
            MessageBox.Show (messageAbout, "A propos de Personal Contact Manager", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
        #endregion

        #region Evénements sur la listBox
        private void listContacts_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Mise à jour de la référence du contact consulté
            contactConsulte = allContacts[listContacts.SelectedIndex];
            afficherContact (contactConsulte);  // Affichage du contact
        }
        #endregion

        #region Evénéments personnalisés
        /// <summary>
        /// Connexion d'un utilisateur authentifié
        /// </summary>
        /// <param name="sender">Emeteur d'un utilisateur valide</param>
        /// <param name="e">Utilisateur authentifié</param>
        private void event_UtilisateurConnecte (Object sender, UserConnectedEventArgs e)
        {
            utilisateurSession = e.ConnectedUser;
            contactSession = new ContactsManager (utilisateurSession);
            isConnected = true;
            updateInterfaceState ();
        }

        private void event_ContactAjoute (Object sender, ContactAddedEventArgs e)
        {
            MessageBox.Show ("Contact Ajouté !", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
            updateInterfaceState ();
        }

        private void event_UtilisateurAjoute(Object sender, UserAddedEventArgs e)
        {
            MessageBox.Show (String.Format ("L'utilisateur {0} à été correctement ajouté(e)", e.AddedUser), "Utilisateur ajouté", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
        #endregion

        #region Evénéments pour un contact donné
        private void btSupprimer_Click (object sender, EventArgs e)
        {
            if (contactSession != null && contactConsulte != null)
            {
                contactSession.Contact = contactConsulte;
                DialogResult reponse = MessageBox.Show ("Voulez vous vraiment supprimer ce contact ?",
                    "Suppression de contact", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (reponse == DialogResult.Yes)
                {
                    contactSession.SupprimerDonnees ();
                    // Suppression du contact de la list des contacts
                    allContacts.RemoveAt (listContacts.SelectedIndex);
                    MessageBox.Show ("Contact Supprimé", "Suppression contact", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    updateInterfaceState ();
                }
            }
        }

        private void btEffacerContact_Click (object sender, EventArgs e)
        {
            txtNom.Text = "";
            txtPrenom.Text = "";
            dpDateNaissance.Value = DateTime.Now;
            txtAdresse.Text = "";
            txtCodePostal.Text = "";
            txtVille.Text = "";
            txtTfixe.Text = "";
            txtTPort.Text = "";
            txtMail.Text = "";
        }

        private void btValiderContact_Click (object sender, EventArgs e)
        {
            if (txtNom.Text.Length < 4 || txtPrenom.Text.Length < 4)
            {
                MessageBox.Show ("Le nom et le prenom du contact doivent faire au moins 4 caractères",
                    "Erreur de saisie", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                if (contactConsulte == null)    // Aucun contact pour cet utilisateur
                {
                    contactConsulte = new Contact (txtNom.Text, txtPrenom.Text,
                    string.Format ("{0}/{1}/{2}", dpDateNaissance.Value.Day.ToString (), dpDateNaissance.Value.Month.ToString (), dpDateNaissance.Value.Year.ToString ()),
                    txtAdresse.Text, txtCodePostal.Text, txtVille.Text, txtTfixe.Text, txtTPort.Text, txtMail.Text);
                    
                    ContactsManager contactManager = new ContactsManager (contactConsulte, utilisateurSession);
                    if (contactManager.AjouterDonnees ())
                    {
                        MessageBox.Show ("Contact Ajouté", "Ajout contact", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                }
                else                        // Les contacts existent
                {
                    contactConsulte = new Contact (contactConsulte.Id, txtNom.Text, txtPrenom.Text,
                        string.Format ("{0}/{1}/{2}", dpDateNaissance.Value.Day.ToString (), dpDateNaissance.Value.Month.ToString (), dpDateNaissance.Value.Year.ToString ()),
                        txtAdresse.Text, txtCodePostal.Text, txtVille.Text, txtTfixe.Text, txtTPort.Text, txtMail.Text);
                }
                contactSession.Contact = contactConsulte;
                contactSession.ModifierDonnees ();
                updateInterfaceState ();
            }
        }
        #endregion
    }
}