﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SQLite;

using PcmCore.Metier;

namespace PcmCore.Donnees
{
    /// <summary>
    /// Cette classe permet de gérer tous les contacts d'un utilisateur
    /// C'est ici que les opérations sur la base de données sont réalisées
    /// </summary>
    public class ContactsManager : AbstractBddManager
    {
        private Contact contact;
        private User utilisateur;
        
        /// <summary>
        /// Contact en cour d'utilisation par le gestionnaire
        /// </summary>
        public Contact Contact
        {
            get { return contact; }
            set { contact = value; }
        }

        /// <summary>
        /// Utilisateur utilisé par le gestionnaire
        /// </summary>
        public User Utilisateur
        {
            get { return utilisateur; }
            set { utilisateur = value; }
        }

        #region Constructeurs

        public ContactsManager (User utilisateur)
        {
            this.contact = null;
            this.utilisateur = utilisateur;
            NomTable = "contact";
        }

        /// <summary>
        /// Construction d'un gestionnaire de contact
        /// </summary>
        /// <param name="contact">Contact sur le quel travailler</param>
        /// <param name="utilisateur">Utilisateur correspondant</param>
        public ContactsManager (Contact contact, User utilisateur) 
            : this(utilisateur)
        {
            this.contact = contact;    
        }

        #endregion

        /// <summary>
        /// Cette méthode permet de récupérer une liste de contact pour 
        /// un utilisateur donné.
        /// </summary>
        /// <returns>Un objet List<Contact> de tous les contacts de l'utilisateur en cour</Contact></returns>
        public List<Contact> AllContacts ()
        {
            List<Contact> contacts = new List<Contact> ();

            SetupConnexion ();
            connexion.Open ();
            SQLiteCommand commande = connexion.CreateCommand ();

            // Requête de vérification
            commande.CommandText = String.Format ("SELECT * FROM {0} WHERE idUtil={1};", NomTable, utilisateur.IdUtilisateur);
            SQLiteDataReader reader = commande.ExecuteReader ();

            while (reader.Read ()) 
            {
                contacts.Add (new Contact (
                    int.Parse(reader["idC"].ToString()),
                    reader["nomC"].ToString (),
                    reader["prenomC"].ToString (),
                    reader["dateAC"].ToString (),
                    reader["adresseC"].ToString (),
                    reader["codePC"].ToString(),
                    reader["villeC"].ToString (),
                    reader["numFixC"].ToString (),
                    reader["numPortC"].ToString (),
                    reader["mailC"].ToString ()));
            }
            connexion.Close ();
            
            return contacts;
        }
        
        /// <summary>
        /// Ajoute un contact à la base de données
        /// Le contact est celui fourni à la construction ou celui fourni manuellement
        /// </summary>
        /// <returns>Retourne True si l'ajout est un succés</returns>
        public override bool AjouterDonnees ()
        {
            bool verification = false;
            if (contact != null)
            {
                SetupConnexion ();
                connexion.Open ();

                // Requête de création d'un utilisateur
                SQLiteCommand commande = connexion.CreateCommand ();
                string insertionUtilisateur = String.Format ("INSERT INTO {0} (nomC, prenomC, dateAC, adresseC, codePC, villeC, numFixC, numPortC, mailC, idUtil) VALUES('{1}', '{2}', '{3}', '{4}', '{5}', '{6}', '{7}', '{8}', '{9}', '{10}');",
                    NomTable, contact.Nom, contact.Prenom, contact.DateNaissance, contact.Adresse, contact.CodePostal, contact.Ville, contact.NumTelFix, contact.NumTelPort, contact.AdrMail, utilisateur.IdUtilisateur);
                commande.CommandText = insertionUtilisateur;

                if (commande.ExecuteNonQuery () != 0)
                    verification = true;

                connexion.Close ();
            }
            return verification;
        }

        /// <summary>
        /// Supprime un contact de la base de données
        /// Le contact est celui fourni à la construction ou celui fourni manuellement
        /// </summary>
        /// <returns>Retourne True si la suppression c'est déroulée correctement sinon False</returns>
        public override bool SupprimerDonnees ()
        {
            bool verification = false;
            if (contact != null)
            {
                SetupConnexion ();
                connexion.Open ();

                // Envoi de la requête de suppression
                SQLiteCommand commande = connexion.CreateCommand ();
                string suppressionContact = String.Format ("DELETE FROM {0} WHERE idC='{1}';",
                    NomTable, contact.Id);
                commande.CommandText = suppressionContact;

                if (commande.ExecuteNonQuery () != 0)
                    verification = true;

                connexion.Close ();
            }
            return verification;
        }

        /// <summary>
        /// Modifie les données d'un contact et les met à jour dans la base de données
        /// Le contact est celui fourni à la construction ou celui fourni manuellement
        /// </summary>
        /// <returns>Retourne True si la modification c'est déroulée correctement sinon False</returns>
        public override bool ModifierDonnees ()
        {
            bool verification = false;
            if (contact != null)
            {
                SetupConnexion ();
                connexion.Open ();

                // Requête de modification d'un contact
                SQLiteCommand commande = connexion.CreateCommand ();
                string modifUtilisateur = String.Format ("UPDATE {0} SET nomC='{1}', prenomC='{2}', dateAC='{3}', adresseC=\"{4}\", codePC='{5}', villeC='{6}', numFixC='{7}', numPortC='{8}', mailC='{9}' WHERE idC={10};",
                    NomTable, contact.Nom, contact.Prenom, contact.DateNaissance, contact.Adresse, contact.CodePostal, contact.Ville, contact.NumTelFix, contact.NumTelPort, contact.AdrMail, contact.Id);

                commande.CommandText = modifUtilisateur;

                // Renvoie 1 si l'ajout est effecté
                if (commande.ExecuteNonQuery () != 0)
                    verification = true;

                connexion.Close ();
            }
            return verification;
        }

        /// <summary>
        /// Cette fonction est chargée de vérifier l'intégrité des données
        /// Elle n'est pas utile dans cette classe.
        /// </summary>
        /// <returns>Renvoie True automatiquement</returns>
        public override int VerifierDonnees ()
        {
            return 1;
        }
        
    }
}
