﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SQLite;

using PcmCore.Metier;

namespace PcmCore.Donnees
{
    public class UserManager : AbstractBddManager
    {
        private User utilisateur;
        private const string nomTableUtilisateur = "utilisateur";

        public User Utilisateur
        {
            get { return utilisateur; }
            set { utilisateur = value; }
        }

        public UserManager ()
        {
            this.utilisateur = null;
         }

        public UserManager (User utilisateur)
        {
            this.utilisateur = utilisateur;
        }

        /// <summary>
        /// Permet de récupérer une collection d'utilisateurs
        /// </summary>
        /// <returns>Tous les utilisateurs enregistrés</returns>
        public List<User> AllUtilisateurs ()
        {
            List<User> listUtilisateurs = new List<User> ();
            SetupConnexion ();
            connexion.Open ();
            SQLiteCommand commande = connexion.CreateCommand ();

            connexion.Close ();

            return listUtilisateurs;
        }

        /// <summary>
        /// Création d'un nouvel utilisateur dans la base de données
        /// </summary>
        /// <returns>
        /// Vrai si l'ajout est un succès
        /// Faux dans le cas contraire
        /// </returns>
        public override bool AjouterDonnees ()
        {
            bool verification = false;

            if (utilisateur != null)
            {
                SetupConnexion ();
                connexion.Open ();

                // Requête de création d'un utilisateur
                SQLiteCommand commande = connexion.CreateCommand ();
                string insertionUtilisateur = String.Format ("INSERT INTO {0} (nomU, passwdU) VALUES('{1}', '{2}');",
                    nomTableUtilisateur, utilisateur.NomUtilisateur, utilisateur.MotDePasse);
                commande.CommandText = insertionUtilisateur;

                // Renvoie 1 si l'ajout est effecté
                int resultatAjout = commande.ExecuteNonQuery ();
                if (resultatAjout != 0)
                    verification = true;

                connexion.Close ();
            }
            return verification;
        }

        /// <summary>
        /// Modification d'un utilisateur
        /// </summary>
        /// <returns>
        /// Vrai si la modification est un succès
        /// Faux dans le cas contraire
        /// </returns>
        public override bool ModifierDonnees ()
        {
            bool verification = false;

            if (utilisateur != null)
            {
                SetupConnexion ();
                connexion.Open ();

                // Requête de création d'un utilisateur
                SQLiteCommand commande = connexion.CreateCommand ();
                string modifUtilisateur = String.Format ("UPDATE {0} SET nomU='{1}', passwdU='{2}' WHERE idU={3};",
                    nomTableUtilisateur, utilisateur.NomUtilisateur, utilisateur.MotDePasse, utilisateur.IdUtilisateur);
                commande.CommandText = modifUtilisateur;

                // Renvoie 1 si l'ajout est effecté
                int resultatModif = commande.ExecuteNonQuery ();
                if (resultatModif != 0)
                    verification = true;

                connexion.Close ();
            }
            return verification;
        }

        /// <summary>
        /// Vérification de la validité d'un utilisateur dans la base de données
        /// </summary>
        /// <returns>
        /// Retourne -1 si l'utilisateur n'existe pas
        /// Sinon retourne l'id de l'utilisateur
        /// </returns>
        public override int VerifierDonnees ()
        {
            int idUtilisateur = -1;

            if (utilisateur != null)
            {
                SetupConnexion ();
                connexion.Open ();
                SQLiteCommand commande = connexion.CreateCommand ();

                // Requête de vérification
                string reqVerif = String.Format ("SELECT * FROM {0} WHERE nomU='{1}' AND passwdU='{2}';",
                    nomTableUtilisateur, utilisateur.NomUtilisateur, utilisateur.MotDePasse);
                commande.CommandText = reqVerif;

                // Authentification
                SQLiteDataReader reader = commande.ExecuteReader ();
                while (reader.Read ()) 
                {
                   if (utilisateur.NomUtilisateur == reader["nomU"].ToString () && utilisateur.MotDePasse == reader["passwdU"].ToString ())
                        idUtilisateur = int.Parse (reader["idU"].ToString ());
                }

                connexion.Close ();
            }
            return idUtilisateur;
        }

        /// <summary>
        /// Supprime un utilisateur de la base de données
        /// </summary>
        /// <returns>
        /// Vrai si la suppresion est un succès
        /// Faux dans le cas contraire
        /// </returns>
        public override bool SupprimerDonnees ()
        {
            bool verification = false;

            if (utilisateur != null)
            {
                SetupConnexion ();
                connexion.Open ();

                // Envoi de la requête de suppression
                SQLiteCommand commande = connexion.CreateCommand ();
                string suppressionUtilisateur = String.Format ("DELETE FROM {0} WHERE idU='{1}';",
                    nomTableUtilisateur, utilisateur.IdUtilisateur);
                commande.CommandText = suppressionUtilisateur;

                // Renvoie 1 si un champ à été modifié
                if (commande.ExecuteNonQuery () != 0) 
                    verification = true;
               
                connexion.Close ();
            }
            return verification;
        }
    }
}