﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PcmCore.Metier
{
    public class Contact
    {
        #region definition des propriétés

        public int Id { get; set; }
        public string Nom { get; set; }
        public string Prenom { get; set; }
        public string DateNaissance { get; set; }
        public string Adresse { get; set; }
        public string CodePostal { get; set; }
        public string Ville { get; set; }
        public string NumTelFix { get; set; }
        public string NumTelPort { get; set; }
        public string AdrMail { get; set; }

        #endregion

        #region Constructeurs

        public Contact ()
        {
            Id = -1;
            Nom = "Nom du contact";
            Prenom = "Prenom du contact";
            DateNaissance = "Date de naissance";
            Adresse = "Adresse du contact";
            CodePostal = "00000";
            Ville = "Ville du contact";
            NumTelFix = "Téléphone fixe";
            NumTelPort = "Téléphone portable";
            AdrMail = "contact@contact.com";
        }

        /// <summary>
        /// Construction d'un nouveau contact ne faisant pas parti de la base de données
        /// </summary>
        /// <param name="nom">Nom du contact</param>
        /// <param name="prenom">Prénom du contact</param>
        /// <param name="dateNaissance">La Date de naissance du contact sous la forme dd/mm/yyyy</param>
        /// <param name="adresse">Adresse du contact</param>
        /// <param name="codePostal">Code Postal du contact</param>
        /// <param name="ville">Ville du contact</param>
        /// <param name="numFix">Numéro de téléphone fixe du contact</param>
        /// <param name="numPort">Numéro de téléphone portable du contact</param>
        /// <param name="adrMail">Adresse email du contact</param>
        public Contact (string nom, string prenom, string dateNaissance, string adresse, string codePostal, string ville, string numFix, string numPort, string adrMail)
        {
            Id = -1;
            Nom = nom;
            Prenom = prenom;
            DateNaissance = dateNaissance;
            Adresse = adresse;
            CodePostal = codePostal;
            Ville = ville;
            NumTelFix = numFix;
            NumTelPort = numPort;
            AdrMail = adrMail;
        }

        /// <summary>
        /// Construction d'un contact existant possédant un numéro d'identification dans la base de données
        /// </summary>
        /// <param name="id">Id du contact</param>
        /// <param name="nom">Nom du contact</param>
        /// <param name="prenom">Prénom du contact</param>
        /// <param name="dateNaissance">La Date de naissance du contact sous la forme dd/mm/yyyy</param>
        /// <param name="adresse">Adresse du contact</param>
        /// <param name="codePostal">Code Postal du contact</param>
        /// <param name="ville">Ville du contact</param>
        /// <param name="numFix">Numéro de téléphone fixe du contact</param>
        /// <param name="numPort">Numéro de téléphone portable du contact</param>
        /// <param name="adrMail">Adresse email du contact</param>
        public Contact (int id, string nom, string prenom, string dateNaissance, string adresse, string codePostal, string ville, string numFix, string numPort, string adrMail)
            : this (nom, prenom, dateNaissance, adresse, codePostal, ville, numFix, numPort, adrMail)
        {
            Id = id;
        }

        #endregion

        public override string ToString ()
        {
            return String.Format ("Nom : {0}\nPrénom : {1}\nDate de naissance : {2}\nAdresse : {3}\n{4} - {5}\nNumero fixe : {6} - portable : {7}\nAdresse eMail : {8}",
                Nom, Prenom, DateNaissance, Adresse, CodePostal, Ville, NumTelFix, NumTelPort, AdrMail);
        }
    }
}
