﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using PcmCore.Metier;
using PcmCore.Event;
using PcmCore.Donnees;

namespace PcmWpf.Gui
{
    /// <summary>
    /// Interaction logic for AddContact.xaml
    /// </summary>
    public partial class AddContactWindow : Window
    {
        public event EventHandler<ContactAddedEventArgs> contactAjoute;
        private User user;

        public AddContactWindow (User user)
        {
            InitializeComponent ();
            this.user = user;
        }

        private void btEffacer_Click(object sender, RoutedEventArgs e)
        {
            txtAdresse.Text = "";
            txtCodePostal.Text = "";
            txteMail.Text = "";
            txtNom.Text = "";
            txtPrenom.Text = "";
            txtTelFixe.Text = "";
            txtTelPort.Text = "";
            txtVille.Text = "";
            if (datePickerNaissance.SelectedDate == null)
                datePickerNaissance.SelectedDate = DateTime.Now;
        }

        private void btEnregistrer_Click(object sender, RoutedEventArgs e)
        {
            if (txtNom.Text.Length < 3 && txtPrenom.Text.Length < 3)
            {
                MessageBox.Show("Le nom et le prénom doivent être composés d'au moins 3 caractères", "Erreur de saisie", MessageBoxButton.OK, MessageBoxImage.Warning);
            }
            else
            {
                if (datePickerNaissance.SelectedDate == null)
                    datePickerNaissance.SelectedDate = DateTime.Now;

                Contact saisieContact = new Contact(
                    txtNom.Text, txtPrenom.Text,
                    string.Format("{0}/{1}/{2}", datePickerNaissance.SelectedDate.Value.Day.ToString(), datePickerNaissance.SelectedDate.Value.Month.ToString(), datePickerNaissance.SelectedDate.Value.Year.ToString()),
                    txtAdresse.Text, txtCodePostal.Text, txtVille.Text, txtTelFixe.Text, txtTelPort.Text, txteMail.Text);
                ContactsManager contactManager = new ContactsManager(saisieContact, user);
                if (contactManager.AjouterDonnees())
                {
                    OnContactAdded(new ContactAddedEventArgs(saisieContact));
                }
                else
                {
                    MessageBox.Show("Le contact n'a pas pu être ajouté", "Erreur", MessageBoxButton.OK, MessageBoxImage.Error);
                }
                this.Close();
            }
        }

        protected virtual void OnContactAdded(ContactAddedEventArgs arg)
        {
            contactAjoute(this, arg);
        }
    }
}
