﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using PcmCore.Metier;
using PcmCore.Donnees;
using PcmCore.Event;

namespace PcmWpf.Gui
{
    /// <summary>
    /// Interaction logic for AddUser.xaml
    /// </summary>
    public partial class AddUserWindow : Window
    {
        public event EventHandler<UserAddedEventArgs> utilisateurAjoute;

        public AddUserWindow ()
        {
            InitializeComponent ();
        }

        private void btAnnuler_Click (object sender, RoutedEventArgs e)
        {
            this.Close ();
        }

        /// <summary>
        /// Ajout d'un contact lors de l'événement click du bouton
        /// On test les deux mots de passe
        /// On test la longueur des mots de passe et nom d'utilisateur
        /// Nom d'utilisateur >= 4 caractères
        /// Mot de passe >= 6 caractères
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btValider_Click (object sender, EventArgs e)
        {
            if (txtUser.Text.Length < 4) MessageBox.Show ("Votre nom d'utilisateur doit contenir au moins 4 caractères", "Erreur de saisie", MessageBoxButton.OK, MessageBoxImage.Exclamation);
            else if (txtPasswd.Password.Length < 6) MessageBox.Show ("Votre mot de passe doit contenir au moins 6 caractères", "Erreur de saisie", MessageBoxButton.OK, MessageBoxImage.Exclamation);
            else if (!(txtPasswd.Password == txtPasswd2.Password)) MessageBox.Show ("Les deux mot de passe ne sont pas identiques", "Erreur de saisie", MessageBoxButton.OK, MessageBoxImage.Exclamation);
            else
            {
                User nouvelUtilisateur = new User (txtUser.Text, txtPasswd.Password);
                UserManager uManager = new UserManager (nouvelUtilisateur);

                if (!uManager.AjouterDonnees ())    // Ajout du nouvel utilisateur à la base de données
                {
                    MessageBoxResult result = MessageBox.Show ("L'utilisateur n'a pas put être ajouté", "Erreur", MessageBoxButton.OKCancel, MessageBoxImage.Error);
                    if (result == MessageBoxResult.Cancel)
                        this.Close ();
                }
                else
                {
                    lancerEventUtilisateurAjoute (new UserAddedEventArgs (nouvelUtilisateur.NomUtilisateur));
                    this.Close ();
                }
            }
        }

        private void lancerEventUtilisateurAjoute (UserAddedEventArgs e)
        {
            utilisateurAjoute (this, e);
        }

        private void Window_KeyDown (object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Escape)
                this.Close ();
        }
    }
}
