﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using PcmCore.Donnees;
using PcmCore.Event;
using PcmCore.Metier;


namespace PcmWpf.Gui
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private bool isConnected;
        private List<Contact> allContacts;
        private Contact currentContact;
        private ContactsManager contactSession;
        private User userSession;

        public MainWindow ()
        {
            this.Content = PcmCore.CoreInformations.CoreVersion;
            isConnected = false;
            allContacts = new List<Contact> ();
            userSession = null;
            contactSession = null;
            currentContact = null;
        }

        /// <summary>
        /// Mise à jour de l'interface suivant l'état de connexion de l'utilisateur
        /// </summary>
        private void updateInterfaceState ()
        {
            if (isConnected)
            {
                itemModifierUtilisateur.IsEnabled = true;
                menuContacts.IsEnabled = true;
                listBoxContacts.Items.Clear();
                allContacts = contactSession.AllContacts();

                foreach (Contact c in allContacts)
                    listBoxContacts.Items.Add(String.Format("{0} {1}", c.Nom, c.Prenom));

                borderDockLeft.Visibility = System.Windows.Visibility.Visible;
                borderDockRight.Visibility = System.Windows.Visibility.Visible;
                etatImage.IsEnabled = true;
                etatPcm.Content = String.Format ("{0} est connecté(e)", userSession.NomUtilisateur);
            }
            else
            {
                userSession = null;
                contactSession = null;
                itemModifierUtilisateur.IsEnabled = false;
                menuContacts.IsEnabled = false;
                borderDockLeft.Visibility = System.Windows.Visibility.Hidden;
                borderDockRight.Visibility = System.Windows.Visibility.Hidden;
                etatImage.IsEnabled = false;
                etatPcm.Content = String.Format("Aucun utilisateur n'est connecté");
                allContacts.Clear ();
            }
        }

        /// <summary>
        /// Affiche un contact dans la zône d'affichage
        /// </summary>
        /// <param name="c">Un objet contact</param>
        private void afficherContact(Contact c)
        {
            int idC = c.Id;
            currentContact = c;
            txtNom.Text = c.Nom;
            txtPrenom.Text = c.Prenom;
            String[] s = c.DateNaissance.ToString().Split('/');
            if (s.Length == 3)
                datePickerNaissance.SelectedDate = new DateTime(int.Parse(s[2]), int.Parse(s[1]), int.Parse(s[0]));
            txtAdresse.Text = c.Adresse;
            txtCodePostal.Text = c.CodePostal;
            txtVille.Text = c.Ville;
            txtTelFixe.Text = c.NumTelFix;
            txtTelPort.Text = c.NumTelPort;
            txtMail.Text = c.AdrMail;
        }

        #region Methodes perso/Custom Methods

        private void connectUser ()
        {
            UserConnectionWindow userConnectionWindow = new UserConnectionWindow ();
            userConnectionWindow.utilisateurConnecte += this.event_UserConnected;
            userConnectionWindow.Show ();
        }

        #endregion

        #region Events Menu Fichier/Files

        private void itemQuit_Click (object sender, RoutedEventArgs e)
        {
            this.Close ();
        }

        private void itemConnexion_Click (object sender, RoutedEventArgs e)
        {
            if (isConnected)    // Si déjà connecté on propose une déconnexion
            {
                MessageBoxResult reponse = MessageBox.Show ("Vous êtes déjà connecté, voulez vous vous déconnecter ?", "Connexion", MessageBoxButton.YesNo, MessageBoxImage.Question);

                if (reponse == MessageBoxResult.Yes)
                {
                    isConnected = false;
                    updateInterfaceState ();
                    connectUser ();
                }
            }
            else
                connectUser ();
        }

        private void itemDeconnexion_Click (object sender, RoutedEventArgs e)
        {
            if (isConnected)
            {
                MessageBoxResult reponse = MessageBox.Show ("Etes vous sur de vous vous déconnecter ?", "Déconnexion", MessageBoxButton.YesNo, MessageBoxImage.Question);
                if (reponse == MessageBoxResult.Yes)
                {
                    isConnected = false;
                    updateInterfaceState ();
                }
            }
            else
                MessageBox.Show ("Aucun utilisateur n'est connecté", "Déconnexion", MessageBoxButton.OK, MessageBoxImage.Exclamation);
        }

        #endregion

        #region Events Menu Utilisateurs/Users

        private void itemAjouterUtilisateur_Click (object sender, RoutedEventArgs e)
        {
            AddUserWindow addUserWindow = new AddUserWindow ();
            addUserWindow.utilisateurAjoute += this.event_UserAdded;
            addUserWindow.Show ();
        }

        private void itemModifierUtilisateur_Click (object sender, RoutedEventArgs e)
        {
            UserEditorWindow userEditorWindow = new UserEditorWindow (userSession);
            userEditorWindow.Show();
        }

        #endregion

        #region Events Menu Contacts

        private void itemAjouterContact_Click (object sender, RoutedEventArgs e)
        {
            AddContactWindow addContactWindow = new AddContactWindow (userSession);
            addContactWindow.contactAjoute += this.event_ContactAdded;
            addContactWindow.Show ();
        }

        /// <summary>
        /// Save contacts in an Xml file
        /// </summary>
        /// <param name="sender">Sender object</param>
        /// <param name="e"></param>
        private void itemSaveToXml_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.SaveFileDialog dialogSave = new Microsoft.Win32.SaveFileDialog();
            dialogSave.DefaultExt = "xml";
            dialogSave.Filter = "Fichier XML (*.xml)|*.xml|Tout types (*.*)|*.*";
            dialogSave.AddExtension = true; // ajoute une extension automatiquement
            dialogSave.RestoreDirectory = true; // Restaure le dernier dossier choisi
            dialogSave.Title = "Où voulez vous sauvegarder vos contacts ?";
            dialogSave.InitialDirectory = @"C:\";

            if (dialogSave.ShowDialog() == true)
            {
                ContactsExporter exporter = new ContactsExporter(contactSession, dialogSave.FileName);
                exporter.ExportToXML();
                if (System.IO.File.Exists(dialogSave.FileName))
                    MessageBox.Show("Vos contacts ont été sauvegardés", "Information", MessageBoxButton.OK, MessageBoxImage.Information);
                else
                    MessageBox.Show("Erreur lors de la sauvegarde de vos contacts", "Erreur", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            dialogSave = null;
        }

        /// <summary>
        /// Save contacts in a text file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void itemSaveToText_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.SaveFileDialog dialogSave = new Microsoft.Win32.SaveFileDialog();
            dialogSave.DefaultExt = "txt";
            dialogSave.Filter = "Fichier Texte (*.txt)|*.txt|Tout types (*.*)|*.*";
            dialogSave.AddExtension = true; // ajoute une extension automatiquement
            dialogSave.RestoreDirectory = true; // Restaure le dernier dossier choisi
            dialogSave.Title = "Où voulez vous sauvegarder vos contacts ?";
            dialogSave.InitialDirectory = @"C:\";

            if (dialogSave.ShowDialog() == true)
            {
                ContactsExporter exporter = new ContactsExporter(contactSession, dialogSave.FileName);
                exporter.ExportToTextFormat();
                if (System.IO.File.Exists(dialogSave.FileName))
                    MessageBox.Show("Vos contacts ont été sauvegardés", "Information", MessageBoxButton.OK, MessageBoxImage.Information);
                else
                    MessageBox.Show("Erreur lors de la sauvegarde de vos contacts", "Erreur", MessageBoxButton.OK, MessageBoxImage.Error);
            }

            dialogSave = null;
        }

        #endregion

        #region Events Menu Aide/Help

        private void itemAproposde_Click (object sender, RoutedEventArgs e)
        {
            MessageBox.Show (PcmCore.CoreInformations.AboutPCM, 
                "A propos de Personal Contacts Manager", MessageBoxButton.OK, MessageBoxImage.Information);
        }

        #endregion

        #region Event Perso/Customs

        // Utilisateur connecté
        private void event_UserConnected (Object sender, UserConnectedEventArgs e)
        {
            userSession = e.ConnectedUser;
            contactSession = new ContactsManager (userSession);
            isConnected = true;
            updateInterfaceState ();
        }

        // Utilisateur ajouté
        private void event_UserAdded (Object sender, UserAddedEventArgs e)
        {
            MessageBox.Show (String.Format ("L'utilisateur {0} à été correctement ajouté(e)", e.AddedUser), "Utilisateur ajouté", MessageBoxButton.OK, MessageBoxImage.Information);
        }

        // Contact ajouté
        private void event_ContactAdded (Object sender, ContactAddedEventArgs e)
        {
            MessageBox.Show ("Contact Ajouté !", "Information", MessageBoxButton.OK, MessageBoxImage.Information);
            updateInterfaceState ();
        }

        #endregion

        #region Events ListBox / Contact's Informations

        private void listContacts_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Mise à jour de la référence du contact consulté
            if (listBoxContacts.SelectedIndex >= 0 && listBoxContacts.SelectedIndex < allContacts.Count)
            {
                currentContact = allContacts[listBoxContacts.SelectedIndex];
                afficherContact(currentContact);  // Affichage du contact
            }
        }

        private void btSupprimer_Click(object sender, RoutedEventArgs e)
        {
            if (contactSession != null && currentContact != null)
            {
                contactSession.Contact = currentContact;
                MessageBoxResult reponse = MessageBox.Show("Voulez vous vraiment supprimer ce contact ?",
                    "Suppression de contact", MessageBoxButton.YesNo, MessageBoxImage.Question);
                if (reponse == MessageBoxResult.Yes)
                {
                    contactSession.SupprimerDonnees();
                    // Suppression du contact de la list des contacts
                    allContacts.RemoveAt(listBoxContacts.SelectedIndex);
                    MessageBox.Show("Contact Supprimé", "Suppression contact", MessageBoxButton.OK, MessageBoxImage.Information);
                    updateInterfaceState();
                }
            }
        }

        private void btEffacer_Click(object sender, RoutedEventArgs e)
        {
            txtNom.Text = "";
            txtPrenom.Text = "";
            datePickerNaissance.SelectedDate = DateTime.Now;
            txtAdresse.Text = "";
            txtCodePostal.Text = "";
            txtVille.Text = "";
            txtTelFixe.Text = "";
            txtTelPort.Text = "";
            txtMail.Text = "";
        }

        private void btEnregistrer_Click(object sender, RoutedEventArgs e)
        {
            if (txtNom.Text.Length < 4 || txtPrenom.Text.Length < 4)
            {
                MessageBox.Show("Le nom et le prenom du contact doivent faire au moins 4 caractères",
                    "Erreur de saisie", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            else
            {
                if (currentContact == null)    // Aucun contact pour cet utilisateur
                {
                    if (datePickerNaissance.SelectedDate == null)
                        datePickerNaissance.SelectedDate = DateTime.Now;

                    currentContact = new Contact(txtNom.Text, txtPrenom.Text,
                    string.Format("{0}/{1}/{2}", datePickerNaissance.SelectedDate.Value.Day.ToString(), datePickerNaissance.SelectedDate.Value.Month.ToString(), datePickerNaissance.SelectedDate.Value.Year.ToString()),
                    txtAdresse.Text, txtCodePostal.Text, txtVille.Text, txtTelFixe.Text, txtTelPort.Text, txtMail.Text);

                    ContactsManager contactManager = new ContactsManager(currentContact, userSession);
                    if (contactManager.AjouterDonnees())
                    {
                        MessageBox.Show("Contact Ajouté", "Ajout contact", MessageBoxButton.OK, MessageBoxImage.Information);
                    }
                }
                else                        // Les contacts existent
                {
                    currentContact = new Contact(currentContact.Id, txtNom.Text, txtPrenom.Text,
                        string.Format("{0}/{1}/{2}", datePickerNaissance.SelectedDate.Value.Day.ToString(), datePickerNaissance.SelectedDate.Value.Month.ToString(), datePickerNaissance.SelectedDate.Value.Year.ToString()),
                        txtAdresse.Text, txtCodePostal.Text, txtVille.Text, txtTelFixe.Text, txtTelPort.Text, txtMail.Text);
                }
                contactSession.Contact = currentContact;
                contactSession.ModifierDonnees();
                updateInterfaceState();
            }
        }
        #endregion

    }
}
